import numpy as np
import math as math
import scipy.signal as sig
#k is the gain at f=0
#f1 is the cuttof frequency in comparison to the nyquist frequency
#the code calculates the bilinear z transform of a transfer function composed out of G*H(s)/G(s) while G(z) and H(z) are
#first Order Butterworth filter with cutoff frequencies at f=f1 and f=f1*k.
#the function returns the coefficients of (b[0]+b[1]*z^-1+b[2]*z^-2)/(a[0]+a[1]*z^-1+b[2]*z^-2)
def Integrator(k,f1):
    if(k<=0):
        raise Exception('gain is zero or is below zero')
    elif(f1<=0):
        raise Exception('cutoff frequency is zero or is below zero')
    elif f1>=1:
        raise Exception('frequency exceeds nyquistfrequency')
    elif k*f1>=1:
        raise Exception('frequency for 0dB cutoff is beyond nyquistfrequency')
    omega=np.zeros(2)
    omega[0]=math.tan(0.5*math.pi*f1)
    omega[1]=math.tan(0.5*math.pi*f1*k)
    b= np.zeros(3)
    a=np.zeros(3)
    a[0]=(1+1/omega[0])
    a[1]=(1-1/omega[0])/a[0]
    b[0]=k*(1+1/omega[1])/a[0]
    b[1]=k*(1-1/omega[1])/a[0]
    a[0]=1.0
    return [b,a]

