import QtQuick 2.7
import QtQuick.LocalStorage 2.0
import QtQuick.Controls 2.2
import QtQuick.Controls.Material 2.2
import LasNQ.ADwin 1.0
import "database.js" as DataBase

Item {
    property alias currentItem: listView.currentItem
    property alias currentIndex: listView.currentIndex
    property int numServos: 8

    function loadServos() {
        if (!numServos)
            return;
        var servos = DataBase.loadServos()
        servoModel.clear()
        for (var j = 1; j <= numServos; j++) {
            servoModel.append({"servoId": j, "servoName": '(Empty Slot)'})
        }
        for (var i = 0; i < servos.length; i++) {
            servoModel.remove(servos[i].id - 1)
            servoModel.insert(servos[i].id - 1, {"servoId": servos[i].id, "servoName": servos[i].name})
        }
        listView.update()
        listView.currentIndex = 0
    }

    onNumServosChanged: {
        loadServos()
    }

    ButtonGroup {
        id: rampGroup1
    }

    ListModel {
        id: servoModel
        ListElement { servoName: 'Dummy Servo'; servoId: 0; servoRamp: false }
    }

    ListView {
        id: listView
        anchors.fill: parent
        model: servoModel
        header: Pane {
            width: parent.width
            height: paneHeader.height + 2*padding
            Material.elevation: 0
            Material.background: "white" //Material.color(Material.Indigo, Material.Shade100)

            z: 10
            PaneHeader {
                id: paneHeader
                width: parent.width
                txtCpuLoad.text: ADwin.cpuLoad + '%'
                txtDeviceNum.text: ADwin.deviceNum
                txtTimeStamp.text: ADwin.seconds
                txtErrorNo.text: ADwin.errorNo
                txtErrorNo.color: ADwin.errorNo ? '#ff0000' : '#555555'
            }
        }
        headerPositioning: ListView.OverlayHeader
        highlight: Rectangle {
            color: Material.color(Material.Indigo, Material.Shade300)
            Rectangle {
                anchors.bottom: parent.bottom
                width: parent.width
                height: 1
                color: Material.color(Material.Indigo, Material.Shade100)
            }
        }
        highlightMoveDuration: 300
        ScrollIndicator.vertical: ScrollIndicator {}
        delegate: ItemDelegate {
            id: servoItem
            width: parent.width
            implicitHeight: 50
            highlighted: index == listView.currentIndex
            onClicked: listView.currentIndex = index
            property variant theModel: model
            contentItem: Rectangle {
                color: 'transparent'
                anchors.fill: parent
                anchors.leftMargin: 5
                Row {
                    RadioButton {
                        width: 40
                        ButtonGroup.group: rampGroup1
                        enabled: false
                        checked: model.servoRamp
                    }
                    /*RadioButton {
                        width: 40
                        ButtonGroup.group: monitorGroup2
                    }*/
                    Label {
                        color: Material.color(Material.Grey, Material.Shade200)
                        height: parent.height
                        verticalAlignment: Text.AlignVCenter
                        text: model.servoName
                    }
                }
            }
        }
    }

    //Component.onCompleted: loadServos()
}

