import QtQuick 2.7
import QtQuick.LocalStorage 2.0
import QtQuick.Controls 2.2
import LasNQ.ADwin 1.0
import "database.js" as DataBase

/*
#Define FCR_SW_INPUT   1
#Define FCR_SW_OUTPUT  2
#Define FCR_SW_RAMP    4
#Define FCR_SW_LIMIT   8
#Define FCR_SW_SOS1    16
#Define FCR_SW_SOS2    32
#Define FCR_SW_SOS3    64
#Define FCR_SW_SOS4    128
#Define FCR_SW_SOS5    256
*/

ServoPaneForm {
    property int servoId: 1
    property int switches: 0
    //property alias ramping: btnRamp

    enabled: false

    function updateFromDB() {
        if (servoId == 0) // this happens on initialisation, just do nothing
            return
        var myServo = DataBase.getServo()
        servoName = myServo.name
        offset = myServo.offset.toFixed(0)
        gain = myServo.gain.toFixed(2)
        switches = myServo.switches
        rampAmpl = myServo.ramp_ampl
        rampFreq = myServo.ramp_freq

        ADwin.activeServo = servoId
        setSwitches()

        reloadFilters(false)
        focus = true
        forceActiveFocus()
        // TODO: ultimately, this also needs to get new values from ADwin?
    }

    function reloadFilters(sendToADwin) {
        var filters = DataBase.getFilters()
        lblFM1.text = filters[0].name
        lblFM2.text = filters[1].name
        lblFM3.text = filters[2].name
        lblFM4.text = filters[3].name
        lblFM5.text = filters[4].name

        if (sendToADwin) {
            // TODO: this then also needs to send filter values to ADwin
        }
    }

    function setSwitches() {
        btnInput.checked = (switches & 1)
        btnOutput.checked = (switches & 2)
        btnOffset.checked = (switches & 8)
        btnFM1.checked = (switches & 16)
        btnFM2.checked = (switches & 32)
        btnFM3.checked = (switches & 64)
        btnFM4.checked = (switches & 128)
        btnFM5.checked = (switches & 256)
        btnRamp.checked = ADwin.hasRamp(servoId)
    }

    function calcSwitchField(bitvalue, checked) {
        if (checked)
            switches = switches | bitvalue
        else
            switches = switches & ~bitvalue
        saveToDB()
    }

    function updateRamp() {
        // update values if ramp active,
        // disable ramp if it is active for current servo, but shouldn't
        if (btnRamp.checked) {
            ADwin.setRamp(servoId, rampFreq, rampAmpl)
        } else if (ADwin.hasRamp(servoId)) {
            ADwin.setRamp(0, 0, 0)
        }
        saveToDB()
    }

    function saveToDB()
    {
        DataBase.saveServo()
    }

    Connections {
        target: ADwin
        onConnected: {
            servoPane.enabled = true
            filterPane.enabled = true
            console.log("Servo enabled")
            updateFromDB()
        }

    }

    //Component.onCompleted: updateFromDB()
    onServoIdChanged: updateFromDB()

    //TODO: not sure why this needs a Connections type, but otherwise I get:
    // ".onEditingFinished" is not available due to component versioning.
    // maybe this is QTBUG-59908
    Connections {
        target: txtServoName
        onEditingFinished: saveToDB()
    }
    Connections {
        target: txtGain
        onEditingFinished: saveToDB()
    }
    Connections {
        target: txtOffset
        onEditingFinished: saveToDB()
    }

    lblRampAmpl.text: (rampAmpl * 20).toFixed(1) + "Vpp"
    lblRampFreq.text: (rampFreq * 0.76).toFixed(1) + "Hz"

    monAux: ADwin.auxMonitor
    monOutput: ADwin.outMonitor
    monInput: ADwin.inMonitor

    btnInput.onClicked: calcSwitchField(1, btnInput.checked)
    btnOutput.onClicked: calcSwitchField(2, btnOutput.checked)
    btnRamp.onClicked: updateRamp()

    btnOffset.onClicked: calcSwitchField(8, btnOffset.checked)

    btnFM1.onClicked: calcSwitchField(16, btnFM1.checked)
    btnFM2.onClicked: calcSwitchField(32, btnFM2.checked)
    btnFM3.onClicked: calcSwitchField(64, btnFM3.checked)
    btnFM4.onClicked: calcSwitchField(128, btnFM4.checked)
    btnFM5.onClicked: calcSwitchField(256, btnFM5.checked)

    Connections {
        target: slRampAmpl
        onMoved: updateRamp()
    }
    Connections {
        target: slRampFreq
        onMoved: updateRamp()
    }

    Keys.onPressed: {
        if (event.key == Qt.Key_Escape)
            focus = true
        else if (event.key == Qt.Key_I) {
            btnInput.toggle(); btnInput.clicked()
        }
        else if (event.key == Qt.Key_O) {
            btnOutput.toggle(); btnOutput.clicked()
        }
        else if (event.key == Qt.Key_Plus) {
            btnOffset.toggle(); btnOffset.clicked()
        }
        else if (event.key == Qt.Key_R) {
            btnRamp.toggle(); btnRamp.clicked()
        }
        else if (event.key == Qt.Key_A) {
            btnAux.toggle(); btnAux.clicked()
        }
    }
    Keys.onDigit1Pressed: { btnFM1.toggle(); btnFM1.clicked() }
    Keys.onDigit2Pressed: { btnFM2.toggle(); btnFM2.clicked() }
    Keys.onDigit3Pressed: { btnFM3.toggle(); btnFM3.clicked() }
    Keys.onDigit4Pressed: { btnFM4.toggle(); btnFM4.clicked() }
    Keys.onDigit5Pressed: { btnFM5.toggle(); btnFM5.clicked() }
    //Keys.onUpPressed: { offset = parseInt(offset) + 100 }
    //Keys.onDownPressed: { offset = parseInt(offset) - 100 }
}
