#ifndef ADWIN_H
#define ADWIN_H

#include <QObject>
#include <QQmlEngine>
#include <QJSEngine>

class ADwinInterface : public QObject
{
    Q_OBJECT
    Q_PROPERTY(int deviceNum READ deviceNum NOTIFY deviceNumChanged)
    Q_PROPERTY(int servoCount READ servoCount NOTIFY servoCountChanged)
    Q_PROPERTY(int seconds READ seconds NOTIFY secondsChanged)
    Q_PROPERTY(int cpuLoad READ cpuLoad NOTIFY cpuLoadChanged)
    Q_PROPERTY(int errorNo READ errorNo NOTIFY errorNoChanged)
    Q_PROPERTY(int inMonitor READ inMonitor NOTIFY inMonitorChanged)
    Q_PROPERTY(int outMonitor READ outMonitor NOTIFY outMonitorChanged)
    Q_PROPERTY(int auxMonitor READ auxMonitor NOTIFY auxMonitorChanged)
    Q_PROPERTY(int activeServo READ activeServo WRITE setActiveServo)

public:
    explicit ADwinInterface(QObject *parent = nullptr);
    ~ADwinInterface() {}

    Q_INVOKABLE bool loadFirmware(int deviceNum, QString firmware);
    Q_INVOKABLE bool runWorker(); // call this often to get new values

    //Q_INVOKABLE void setMonitor(int monitor, int servo);
    //Q_INVOKABLE int getMonitor(int monitor);

    //Q_INVOKABLE void setServo(int servo, int fcr);
    //Q_INVOKABLE int getServo(int servo);

    //Q_INVOKABLE void setCoeffs(int servo, QVariantList coeffs);
    //Q_INVOKABLE QVariantList getCoeffs(int servo);

    Q_INVOKABLE void setRamp(int servo, int freq, double ampl);
    Q_INVOKABLE bool hasRamp(int servo);

    int deviceNum() const { return m_deviceNum; }

    int servoCount() const { return m_servoCount; }
    void setServoCount(int val) { m_servoCount = val; emit servoCountChanged(val); }

    int seconds() const { return m_seconds; }
    void setSeconds(int val) { m_seconds = val; emit secondsChanged(val); }

    int cpuLoad() const { return m_cpuLoad; }
    void setCpuLoad(int val) { m_cpuLoad = val; emit cpuLoadChanged(val); }

    int errorNo() const { return m_errorNo; }
    void setErrorNo(int val) { m_errorNo = val; emit errorNoChanged(val); }

    int inMonitor() const { return m_monitor[0]; }
    int auxMonitor() const { return m_monitor[1]; }
    int outMonitor() const { return m_monitor[2]; }
    int activeServo() const { return m_activeServo; }
    void setActiveServo(int val) { m_activeServo = val; }

signals:
    void deviceNumChanged(int newValue);
    void servoCountChanged(int newValue);
    void secondsChanged(int newValue);
    void cpuLoadChanged(int newValue);
    void errorNoChanged(int newValue);
    void inMonitorChanged(int newValue);
    void auxMonitorChanged(int newValue);
    void outMonitorChanged(int newValue);
    void connected();

private:
    int m_deviceNum;
    bool m_booted;

    int m_seconds;
    int m_cpuLoad;
    int m_servoCount;
    long m_errorNo;

    int m_monitor[3];
    int m_activeServo;
};

QObject *adwin_interface_provider(QQmlEngine *engine, QJSEngine *scriptEngine);

#endif // SINGLETONTYPEEXAMPLE_H
