import QtQuick 2.7
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import QtQuick.Controls.Material 2.2
import LasNQ.ADwin 1.0

ApplicationWindow {
    id: window
    visible: true
    width: 1000
    height: 600
    title: qsTr("Servo Controller")
    minimumWidth: 1000

    Material.theme: Material.Light
    Material.accent: '#CB0B31'

    Drawer {
        id: drawer
        width: 250
        height: window.height
        y: 0; z: 2
        Material.elevation: 5
        modal: false
        visible: true
        position: 1
        padding: 0
        interactive: false
        Pane {
            anchors.margins: 0
            anchors.fill: parent
            padding: 0
            Material.background: Material.Indigo

            ServoList {
                id: servoList
                numServos: ADwin.servoCount
                anchors.fill: parent
            }
        }

    }

    SwipeView {
        id: swipe
        currentIndex: page_indicator.currentIndex
        anchors.leftMargin: drawer.width
        anchors.fill: parent

        ServoPane {
            id: servoPane
            servoId: servoList.currentItem.theModel.servoId
            onServoNameChanged: servoList.currentItem.theModel.servoName = servoName
            btnRamp.onClicked: servoList.currentItem.theModel.servoRamp = btnRamp.checked
        }

        FilterPane {
            id: filterPane
            servoName: servoList.currentItem.theModel.servoName
            servoId: servoList.currentItem.theModel.servoId
            onCoeffsChanged: servoPane.reloadFilters(true)
        }
    }

    PageIndicator {
        id: page_indicator
        count: swipe.count
        currentIndex: swipe.currentIndex
        interactive: true

        anchors.bottom: swipe.bottom
        anchors.horizontalCenter: swipe.horizontalCenter
    }

    Timer {
        id: updateTimer
        interval: 200
        repeat: true
        running: false
        onTriggered: ADwin.runWorker()
    }

    Component.onCompleted: ADwin.loadFirmware(1, "c:\\Users\\adwin\\documents\\adwin\\adwin-control\\adwin-control.tc1")
    Connections {
        target: ADwin
        onConnected: updateTimer.start()
    }
}
